<?php

namespace SupplyTek\Controllers;

use SupplyTek\Utilities\EmailHandler;
use SupplyTek\Utilities\TemplateHandler;
use SupplyTek\Services\ApprovalWorkflowService;

/**
 * Handles Approval/Disapproval Workflow from SupplyTek
 * 
 * Controls the actions needed for each workflow and send
 * emails to users with notifications of the status being
 * updated.
 *
 * @author Rômulo Alves Lousada
 */
class ApprovalWorkflowController
{
    protected $email;
    protected $template;
    protected $service;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param EmailHandler $emailHandler Object to access EmailController
     * @param TemplateHandler $templateHandler Object that allows to retrive templates
     * @param ApprovalWorkflowService $approvalWorkflowService Object to access ApprovalWorkflowService
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct(EmailHandler $emailHandler, TemplateHandler $templateHandler, ApprovalWorkflowService $approvalWorkflowService)
    {
        $this->email = $emailHandler;
        $this->template = $templateHandler;
        $this->service = $approvalWorkflowService;
    }
    
    /**************************************************************************/
    /*                             WORKFLOW START                             */
    /**************************************************************************/
    
    /**
     * Initial function to start the workflow process
     * 
     * @param array $data Array with initial data needed to start the process
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function startWorkflow($data)
    {
        try {
            $data["P_NRCONFIGALCADA"] = $this->service->getAuthorityNumber($data);
            
            if ($data["P_NRCONFIGALCADA"]) {
                $data["P_NRAPROVALCADA"] = $this->service->saveApprovalRegistry($data);
                $this->service->updateAFWithAuthorityData($data);
                
                $user = $this->service->getUserData($data);
                $mail = $this->service->getMailData($data);
                
                if ($user && $mail) {
                    $messages = $this->createMessagesRequesterApproval($data, $user, $mail);
                    //$this->email->send($messages["SUBJECT"], $messages["CONTENT"], $messages["RECIPIENT"]);
                } else {
                    throw new \Exception("Dados insuficientes para enviar o email.", 500);
                }
            } else {
                throw new \Exception("Alçada não encontrada.", 500);
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Creates an array with data needed on the mail
     * 
     * @param array $data Array with initial data needed to start the process
     * @param array $user Array with user data needed on the process
     * @param array $mail Array with mail data needed on the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createMessagesRequesterApproval($data, $user, $mail)
    {
        $mail["DTAUTOFORN"] = substr($mail["DTAUTOFORN"], 0, 10);
        $mail["DTPROGENTAF"] = substr($mail["DTPROGENTAF"], 0, 10);
        
        $file = array("name" => "requester-approval", "extension" => ".html");

        $supplyTekUrl = "http://" . $_SERVER['HTTP_HOST'];
        $replace = $this->getReplaceArray($data, $user, $mail, $supplyTekUrl);

        $messages = array();
        $messages["RECIPIENT"] = $user["DSEMAILOPER"];
        $messages["SUBJECT"] = "Aprovação Pendente - Nº " . $data["P_NRDEMANDA"];
        $messages["CONTENT"] = $this->template->getTemplate($file, $replace);
                        		
        return $messages;
    }
    
    /**
     * Return the search-replace array on the mail.
     * 
     * @param array $data Array with initial data needed to start the process
     * @param array $user Array with user data needed on the process
     * @param array $mail Array with mail data needed on the process
     * @param string $supplyTekUrl String with SupplyTek's url
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getReplaceArray($data, $user, $mail, $supplyTekUrl)
    {
        return array(
            "%OPERATOR_NAME%" => $user["NMOPERADOR"], 
            "%ORDER_NUMBER%" => $data["P_NRDEMANDA"], 
            "%UNIT_ID%" => $mail["CDFILDEST"], 
            "%UNIT_NAME%" => $mail["NMFILDEST"], 
            "%AF_DATE%" => $mail["DTAUTOFORN"], 
            "%AF_PROG_DATE%" => $mail["DTPROGENTAF"], 
            "%AF_TOTAL_VALUE%" => $mail["VRTOTALAF"], 
            "%BUYER_NAME%" => $mail["COMPRADOR"], 
            "%SUPPLIER_REGISTER%" => $mail["NRINSJURFORN"],
            "%SUPPLIER_NAME%" => $mail["NMFANTFORN"],
            "%REQUESTER_NAME%" => $mail["NMSOLICITANTE"],
            "%REQUESTER_EMAIL%" => $mail["EMAILSOLICITANTE"],
            "%SUPPLYTEK_URL%" => $supplyTekUrl
        );
    }
}