<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\CrudSupplyTek;
use SupplyTek\Utilities\SessionHandler;
use SupplyTek\Services\NotificationsService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;


/**
 * Search for notifications information.
 *
 * @author Bruna Chaves Carneiro
 */
class NotificationsController 
{
    protected $crud;
    protected $session;
    protected $service;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param CrudSupplyTek $crudSupplyTek Object to access CrudSupply functions
     * @param SessionHandler $sessionHandler Object that allows to manipulate PHP Session
     * @param NotificationsService $notificationsService Object to access NotificationsService
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct (CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler, NotificationsService $notificationsService) 
    {
        $this->crud = $crudSupplyTek;
        $this->session = $sessionHandler;
        $this->service = $notificationsService;
    }

    /**
     * Returns user notifications
     * 
     * Returns notifications according to the unit and the user has logged in to the system.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function getNotifications(Request $request, Response $response)
    {   
        try {
            $cdFilial = $this->session->getSessionData('CDFILIAL');
            $cdOperador = $this->session->getSessionData('CDOPERADOR');
            
            $params = array(
                "P_CDFILIAL"   => $cdFilial,
                "P_CDOPERADOR" => $cdOperador
            );
            
            $notifications = $this->crud->find($request, $params);

            $response->setMetadata($notifications);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    /**
     * Mark notifications as read
     * 
     * Receive the parameters in the request and send to the service to update 
     * the selected notification.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function markNotificationsRead(Request $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            
            if (isset($rows[0])) {
                $rows[0]["READ"] = $rows[0]["READ"] == true ? "S" : "N";
                $rows[0]["UNIT_ID"] = $this->session->getSessionData("CDFILIAL");
                $rows[0]["OPERATOR_ID"] = $this->session->getSessionData("CDOPERADOR");
                
                $this->service->updateNotifications($rows[0]);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Delete notifications
     * 
     * Receive the parameters in the request and send to the service to delete 
     * the selected notifications.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     * 
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function deleteNotifications(Request $request, Response $response)
    {
        try {
            $parameters = $request->getParameters();
            $notificationId = isset($parameters['ID']) ? $parameters['ID'] : false;
            
            $this->service->beginTransaction();
            
            if ($notificationId) {
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                
                $this->service->deleteNotifications($notificationId, $operatorId, $unitId);
                
                $this->service->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->service->rollback();
            throw $e;
        }
    }
}